#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "main.h"

/*  Declare Windows procedure  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);

/*  Make the class name into a global variable  */
char szClassName[ ] = "WindowsApp";
HWND hStatusBar, hStatusButton;
HINSTANCE hInstance;

int WINAPI WinMain (HINSTANCE hThisInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszArgument, int nFunsterStil)

{
    HWND hwnd;               /* This is the handle for our window */
    MSG messages;            /* Here messages to the application are saved */
    WNDCLASSEX wincl;        /* Data structure for the window~class */

    /* The Window structure */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc = WindowProcedure;      /* This function is called by windows */
    wincl.style = CS_DBLCLKS;                 /* Catch double-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL;                 /* No menu */
    wincl.cbClsExtra = 0;                      /* No extra bytes after the window class */
    wincl.cbWndExtra = 0;                      /* structure or the window instance */
    /* Use Windows's default color as the background of the window */
    wincl.hbrBackground = (HBRUSH) COLOR_BACKGROUND;

    /* Register the window class, and if it fails quit the program */
    if (!RegisterClassEx (&wincl))
        return 0;

    /* The class is registered, let's create the program*/
    hwnd = CreateWindowEx (
           WS_EX_CLIENTEDGE,    /* Extended possibilites for variation */
           szClassName,         /* Classname */
           "Practicando Windows",  /* Title Text */
           WS_OVERLAPPEDWINDOW, /* default window */
           CW_USEDEFAULT,       /* Windows decides the position */
           CW_USEDEFAULT,       /* where the window ends up on the screen */
           644,                 /* The programs width */
           475,                 /* and height in pixels */
           HWND_DESKTOP,        /* The window is a child-window to desktop */
           NULL,                /* No menu */
           hThisInstance,       /* Program Instance handler */
           NULL                 /* No Window Creation data */
           );

    /* Make the window visible on the screen */
    WinMenu (hwnd);
    ShowWindow (hwnd, nFunsterStil);

    /* Run the message loop. It will run until GetMessage() returns 0 */
    while (GetMessage (&messages, NULL, 0, 0))
    {
        /* Translate virtual-key messages into character messages */
        TranslateMessage(&messages);
        /* Send message to WindowProcedure */
        DispatchMessage(&messages);
    }

    /* The program return-value is 0 - The value that PostQuitMessage() gave */
    return messages.wParam;
}


/*  This function is called by the Windows function DispatchMessage()  */

LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    int iStatusWidths[ ] = {70, 300, 350, 400, -1}; /* medidas de cada parte */
    static HFONT hfont;/* fuente del boton */
    
    switch (message)                  /* handle the messages */
    {
        case WM_CREATE:
        {    
            /* Creamos la barra de estado */
            hStatusBar = CreateWindowEx(0, STATUSCLASSNAME, NULL, 
               WS_VISIBLE | WS_CHILD | SBARS_SIZEGRIP, 0, 0, 0, 0, hwnd, 
               (HMENU)ID_STATUS, hInstance, NULL);
            /* Envia un mensaje para definir en cuntas partes se va a dividir */
            SendMessage(hStatusBar, SB_SETPARTS, 5, (LPARAM)iStatusWidths);
            /* Muestra un texto */
            SendMessage(hStatusBar, SB_SETTEXT, 1, (LPARAM)"Listo");
            /* Creamos un boton, hijo de la barra de estado */   
            hStatusButton = CreateWindowEx(0, "BUTTON", "Saludo", WS_VISIBLE | WS_CHILD,
               0, 2, 71, 22, hStatusBar, (HMENU)ID_STATUSBUTTON, hInstance, NULL);
            /* Lo enviamos a esta */
            SendMessage(hStatusBar, ID_STATUSBUTTON, 2, 0);
            /* Cambiamos su diseo de fuente */
            hfont = CreateFont(12, 0, 0, 0, 300, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
              OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, DEFAULT_PITCH | FF_ROMAN, "Helv");
            SendMessage(hStatusButton, WM_SETFONT, (WPARAM)hfont, MAKELPARAM(TRUE, 0));
            break;
        }
        case WM_SIZE:
        {
            /* Para redimensionar la Barra de estado cuandio extendemos la ventana */
            RECT rectClient, rectStatus, rectTool;
            UINT uToolHeight, uStatusHeight, uClientAlreaHeight;
            /* Envia el mensaje del tamao al manipulador de la barra de estado */
            SendMessage(hStatusBar, WM_SIZE, 0, 0);
            /* Toma las medidas actuales */
            GetClientRect(hwnd, &rectClient);
            GetWindowRect(hStatusBar, &rectStatus);
            /* Calcula cunto hay que modificar */
            uStatusHeight = rectStatus.bottom - rectStatus.top;
            uClientAlreaHeight = rectClient.bottom;
            /* Mueve los elementos segun movamos la ventana */
            MoveWindow((HWND)szClassName, 0, uToolHeight, rectClient.right, uClientAlreaHeight - uStatusHeight - uToolHeight, TRUE);
        }
        break;
        case WM_COMMAND:
        {
            switch (LOWORD(wParam))
            {
                case ID_STATUSBUTTON:/* Comando del boton "Saludo" */
                    MessageBox(hStatusBar, "Has pulsado este botn", "", MB_OK | MB_ICONINFORMATION);
                    break;
                case CM_QUIT:
                    int mb = MessageBoxEx(hwnd, "Deseas salir?", "", MB_YESNO | MB_ICONINFORMATION, 0);
                    if (mb == IDYES) PostMessage(hwnd, WM_CLOSE, 0, 0);
                    else return 1;
                    break;
            }
            break;
        }
        case WM_CLOSE:
            DestroyWindow(hwnd);
            break;
        case WM_DESTROY:
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            break;
        default:                      /* for messages that we don't deal with */
            return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return 0;
}

void WinMenu (HWND hwnd)
{
    HMENU menubar, hmenu1;
    menubar = CreateMenu();
    AppendMenu(menubar, MF_STRING, CM_QUIT, "Salir");
    SetMenu (hwnd, menubar);
}
